/*
    File name: easywiz.c
    Author: axelorque
    Date: 15/03/2010
    Last update: 04/09/2010
    
    Contain functions and globals ressources to make applications on GP2X Wiz.
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <dirent.h>
#include <stdarg.h>
#include <SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include "easywiz.h"

/* The screen surface, joystick device, font, color */
SDL_Surface *screen = NULL;
SDL_Surface *tmp_screen = NULL;
SDL_Joystick *joy = NULL;
TTF_Font *font = NULL;
SDL_Color color = {255, 255, 255};

wiz_image image[MAX_IMAGES];

/*##############################################################################
#                         Init  & Info Functions                               #
##############################################################################*/
/* wiz init */
bool wiz_Init(void) 
{
    /* Initialize SDL's Subsystems */ 
	if (SDL_Init (SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK | SDL_INIT_TIMER) < 0) {
		fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
		return false;
	}	

	/* Set 320x240 16-bits video mode */
	screen = SDL_SetVideoMode(WIZ_WIDTH, WIZ_HEIGHT, WIZ_DEPTH, SDL_SWSURFACE|SDL_DOUBLEBUF);
	if (screen == NULL) {
		fprintf (stderr, "Couldn't set 320x240x16 video mode: %s\n", SDL_GetError ());
		return false;
	}
																					/* rrrr rggg gggb bbbb no alpha */
	tmp_screen = SDL_CreateRGBSurface(SDL_SWSURFACE, WIZ_WIDTH, WIZ_HEIGHT, WIZ_DEPTH, 0xf800, 0x07e0, 0x001f, 0x0000);

    SDL_ShowCursor(SDL_DISABLE);
	/* Check and open joystick device */
	joy = SDL_JoystickOpen(0);
	if(!joy) {
		fprintf (stderr, "Couldn't open joystick 0: %s\n", SDL_GetError ());
		return false;
	}
	
	/* Initialize SDL_ttf */ 
	if(TTF_Init() == -1) {
        fprintf (stderr, "Couldn't initialize TTF: %s\n", TTF_GetError ());
        return false;
    }
    font = TTF_OpenFont("/usr/gp2x/HYUni_GPH_B_V1.01.ttf", 11);
    
    atexit(wiz_Terminate);
	return true;
}

/* Release open systems and ressources */
static void wiz_Terminate(void)
{
    wiz_FreeImage(); 
    SDL_FreeSurface(tmp_screen);
    SDL_JoystickClose(joy);
    TTF_CloseFont(font);
    TTF_Quit();
    SDL_Quit();
    chdir("/usr/gp2x");
    execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
}

/* Display a splash screen (not really interesting :p) */
void wiz_SplashScreen(const char *name, Uint32 time)
{
    int i = 0; 
    SDL_Surface *tmp = IMG_Load(name);
    SDL_Surface *img = SDL_DisplayFormatAlpha(tmp);
    SDL_FreeSurface(tmp);
    
    if(img->w != 320 || img->h != 240)
    {
        SDL_Rect dst;
        dst.x = 160 - img->w/2;
        dst.y = 120 - img->h/2;
        SDL_BlitSurface(img, 0, tmp_screen, &dst);
    }
    else
        SDL_BlitSurface(img, 0, tmp_screen, 0);    
    SDL_FreeSurface(img);
    wiz_UpdateScreen();
    SDL_Delay(time);
    
    tmp = SDL_CreateRGBSurface(SDL_SWSURFACE, WIZ_WIDTH, WIZ_HEIGHT, WIZ_DEPTH, 0xf800, 0x07e0, 0x001f, 0x0000);
    
    for(i = 0; i < 128; i+=5)
    {
        SDL_SetAlpha(tmp, SDL_SRCALPHA, i);  
        SDL_BlitSurface(tmp, 0, tmp_screen, 0);
        SDL_Delay(60);
        wiz_UpdateScreen();
    }
    SDL_FreeSurface(tmp);
}

/* Return a value between 1 and 4
   (1: battery full charged. 4: battery Low */
unsigned short wiz_GetBatteryState(void)
{
    int fd = open("/dev/pollux_batt", O_RDONLY);
    unsigned short val = 0;

    if(fd >= 0)
    {
        read(fd, &val, sizeof(unsigned short));
        close(fd);
    }

    return val;
}

/*##############################################################################
#                        Input-Output Function                                 #
##############################################################################*/
/* Event management */
bool wiz_UpdateInput(Input *in)
{
    bool ret = false;
    SDL_Event event;
    while(SDL_PollEvent(&event))
    {
        switch(event.type)
        {
            case SDL_JOYBUTTONDOWN:
                in->key[event.jbutton.button] = 1;
                ret = true;
            break;
            case SDL_JOYBUTTONUP:
                in->key[event.jbutton.button] = 0; 
            break;
            case SDL_MOUSEMOTION:
                in->mousex = event.motion.x;
                in->mousey = event.motion.y;
                in->mousexrel = event.motion.xrel;
                in->mouseyrel = event.motion.yrel;
            break;
            case SDL_MOUSEBUTTONDOWN:
                in->mousebuttons[event.button.button-1] = 1; 
                ret = true;
            break;
            case SDL_MOUSEBUTTONUP:
                in->mousebuttons[event.button.button-1] = 0; 
            break;
            default:
            break;
        }
    }
    return ret;
}

/* Flip screen */
void wiz_UpdateScreen(void) 
{
	SDL_BlitSurface(tmp_screen, 0, screen, 0);
	SDL_Flip(screen);
}

/* lock framerate */
void wiz_framerate(Uint32 fps)
{
     static Uint32 next_time = 0;
     Uint32 now;
     
     now = SDL_GetTicks();
     if(next_time <= now)
         next_time = now + (1000 / fps);
     else
         SDL_Delay(next_time - now);
}

/* switch on (true) or switch off (false) the screen */
bool wiz_SetScreenOnOff(bool value)
{
    unsigned int send[2];
    int fd = open("/dev/fb0", O_RDWR);

    if(fd >= 0)
    {
        send[0] = (value) ? 1 : 0;
        send[1] = 0;
        ioctl(fd, _IOW('D', 90, unsigned int[2]), &send);
        close(fd);
        return send[0];
    }
    
    return !value;
}

/*##############################################################################
#                         Image Functions                                      #
##############################################################################*/
/* Load an image */
bool wiz_LoadImage(Uint8 n, const char *name)
{
    bool ret = true;
    SDL_Surface *tmp = IMG_Load(name);
    ret = wiz_LoadSurface(n, tmp);
    SDL_FreeSurface(tmp);
    return ret;
}

/* Load a SDL_Surface */
bool wiz_LoadSurface(Uint8 n, SDL_Surface *tmp) 
{
    if(n >= MAX_IMAGES) return false;
    if(image[n].img) // vite les fuites mmoires
        SDL_FreeSurface(image[n].img);
    image[n].img = SDL_DisplayFormatAlpha(tmp);
    return true;
}

/* Display an image */
void wiz_DisplayImage(Uint8 n, int x, int y) 
{
    if(!image[n].img) return;
    SDL_Rect dst;
    dst.w = image[n].img->w;
    dst.h = image[n].img->h;
    dst.x = x;
    dst.y = y;
    SDL_BlitSurface(image[n].img, 0, tmp_screen, &dst);
}

/* Display a part of an image */
void wiz_DisplayImagePart(Uint8 n, int x, int y, int w, int h, int posx, int posy) 
{
    if(!image[n].img) return;
    SDL_Rect dst;    
    SDL_Rect src;
    dst.w = w;
    dst.h = h;
    dst.x = x;
    dst.y = y;
    src.w = w;
    src.h = h;
    src.x = posx;
    src.y = posy;
    SDL_BlitSurface(image[n].img, &src, tmp_screen, &dst);
}

/* Release all images */
static void wiz_FreeImage(void)
{
     int i = 0;
     for(i = 0; i < MAX_IMAGES; i++)
     {
         if(image[i].img)
             SDL_FreeSurface(image[i].img);
     }
}

/* Take a screenshot */
void wiz_Screenshot(void)
{
    int nbr = 0;
    char str[25];
    struct dirent *ent = NULL;
    DIR* rep = NULL;
    
    if((rep = opendir("./screenshot")) == NULL)
    {
        mkdir("screenshot", 0777);
        rep = opendir("./screenshot");
    }
    
    while((ent = readdir(rep)) != NULL)
    {
        if(strcmp(ent->d_name, ".") != 0 && strcmp(ent->d_name, "..") != 0)
            nbr++;
    }
    
    if(closedir(rep) == -1)
        return;
    
    sprintf(str, "./screenshot/img_%d.bmp", nbr);
    SDL_SaveBMP(screen, str);
}

/*##############################################################################
#                          Text Functions                                      #
##############################################################################*/
/* Load a text surface */
bool wiz_LoadTextSurface(Uint8 n, const char *text)
{
    if(n >= MAX_IMAGES) return false; 
    SDL_Surface *message = TTF_RenderText_Solid(font, text, color);
    if(message == NULL)
        return false;
    if(!wiz_LoadSurface(n, message))
    {
        SDL_FreeSurface(message);
        return false;
    }
    SDL_FreeSurface(message);
    return true;
}


